@@ -56,12 +56,14 @@ class ShellTest(ProcessTest, SiteTest, unittest.TestCase):
 
     @defer.inlineCallbacks
     def test_redirect_not_follow_302(self):
-        _, out, _ = yield self.execute(['--no-redirect', self.url('/redirect-no-meta-refresh'), '-c', 'response.status'])
+        _, out, _ = yield self.execute(
+            ['--no-redirect', self.url('/redirect-no-meta-refresh'), '-c', 'response.status']
+        )
         assert out.strip().endswith(b'302')
 
     @defer.inlineCallbacks
     def test_fetch_redirect_follow_302(self):
-        """Test that calling `fetch(url)` follows HTTP redirects by default."""
+        """Test that calling ``fetch(url)`` follows HTTP redirects by default."""
         url = self.url('/redirect-no-meta-refresh')
         code = "fetch('{0}')"
         errcode, out, errout = yield self.execute(['-c', code.format(url)])
@@ -71,7 +73,7 @@ class ShellTest(ProcessTest, SiteTest, unittest.TestCase):
 
     @defer.inlineCallbacks
     def test_fetch_redirect_not_follow_302(self):
-        """Test that calling `fetch(url, redirect=False)` disables automatic redirects."""
+        """Test that calling ``fetch(url, redirect=False)`` disables automatic redirects."""
         url = self.url('/redirect-no-meta-refresh')
         code = "fetch('{0}', redirect=False)"
         errcode, out, errout = yield self.execute(['-c', code.format(url)])
@@ -101,15 +103,13 @@ class ShellTest(ProcessTest, SiteTest, unittest.TestCase):
     @defer.inlineCallbacks
     def test_local_nofile(self):
         filepath = 'file:///tests/sample_data/test_site/nothinghere.html'
-        errcode, out, err = yield self.execute([filepath, '-c', 'item'],
-                                       check_code=False)
+        errcode, out, err = yield self.execute([filepath, '-c', 'item'], check_code=False)
         self.assertEqual(errcode, 1, out or err)
         self.assertIn(b'No such file or directory', err)
 
     @defer.inlineCallbacks
     def test_dns_failures(self):
         url = 'www.somedomainthatdoesntexi.st'
-        errcode, out, err = yield self.execute([url, '-c', 'item'],
-                                       check_code=False)
+        errcode, out, err = yield self.execute([url, '-c', 'item'], check_code=False)
         self.assertEqual(errcode, 1, out or err)
         self.assertIn(b'DNS lookup failed', err)
